#include <iostream>

#include "fitter.hpp"
#include "likelihoodcomponents.hpp"
#include "comparison.hpp"

using namespace myfitter;


bool differ(double a, double b)
{
    return differ(a, b, 1.e-5, 1.e-10);
}



enum Par {P1, P2, P3, P4, NPAR};
enum Obs {O1, O2, O3, O4, O5, O6, O7, NOBS};

int main()
{
    CorrelatedGaussianLC lc(NOBS);
    lc.add(O3, 1.4, 0.2);
    lc.add(O5, -0.7); lc.sigma(O5, 0.1);
    lc.add(O7); lc.central_value(O7, 0.9); lc.sigma(O7, 0.3);

    lc.cor(O3, O5, -0.1);
    lc.cor(O3, O7, -0.8);
    lc.cor(O5, O7, +0.3);

    // this should have no effect
    lc.cor(O3, O3, -0.5);

    // this will change cor(O3, O7)
    lc.cor(O7, O3, -0.5);


    ObservableVector cvals(NOBS);
    lc.get_central_values(cvals);

    ObservableVector obs(NOBS);
    obs(O1) = Model::nan();
    obs(O2) = Model::nan();
    obs(O3) = +1.7;
    obs(O4) = Model::nan();
    obs(O5) = -0.5;
    obs(O6) = Model::nan();
    obs(O7) = +1.0;

    Matrix hess(NOBS, NOBS, Model::nan());
    lc.get_hessian(hess, obs);

    if(differ(hess(O3,O3), +33.4559)) return 1;
    if(differ(hess(O3,O5), -3.67647)) return 1;
    if(differ(hess(O3,O7), +11.5196)) return 1;

    if(differ(hess(O5,O3), -3.67647)) return 1;
    if(differ(hess(O5,O5), +110.294)) return 1;
    if(differ(hess(O5,O7), -12.2549)) return 1;

    if(differ(hess(O7,O3), +11.5196)) return 1;
    if(differ(hess(O7,O5), -12.2549)) return 1;
    if(differ(hess(O7,O7), +16.1765)) return 1;

    if(differ(lc.calc(cvals,obs)-lc.calc(cvals,cvals), 7.34436)) return 1;

    return 0;
}
